#include "TFT_objects.h"
#include "TFT_resources.h"
#include "built_in.h"


// TFT module connections
sbit TFT_BLED at GPIOF_ODR.B10;
sbit TFT_CS   at GPIOF_ODR.B12;
unsigned int TFT_DataPort at GPIOG_ODR;
sbit TFT_RD   at GPIOF_ODR.B14;
sbit TFT_RS   at GPIOF_ODR.B13;
sbit TFT_RST  at GPIOF_ODR.B11;
sbit TFT_WR   at GPIOF_ODR.B15;
// End TFT module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TImage *local_image;
TImage *exec_image;
int image_order;
TCImage *local_cimage;
TCImage *exec_cimage;
int cimage_order;


void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  GPIOG_ODR = (unsigned int) (_lo | (_hi << 8));
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Write_to_Data_Lines(Hi(_data), Lo(_data));
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

static void InitializeTouchPanel() {
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_SSD1963_Board_50(800, 480);

  TFT_Set_DBC_SSD1963(255);

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;

  TScreen                Screen2;
  TCImage               Image1 =
         {
         &Screen2              , //   Image1.OwnerScreen
         0                     , //   Image1.Order
         0                     , //   Image1.Left
         0                     , //   Image1.Top
         800                   , //   Image1.Width
         480                   , //   Image1.Height
         &screen_2_bckg_bmp    , //   Image1.Picture_Name
         1                     , //   Image1.Visible
         1                     , //   Image1.Active
         0                     , //   Image1.Picture_Type
         1                     , //   Image1.Picture_Ratio
         0                     , //   Image1.OnUpPtr
         0                     , //   Image1.OnDownPtr
         0                     , //   Image1.OnClickPtr
         0                       //   Image1.OnPressPtr
         };
  TImage               Image2;
  TImage               Image3;
  TImage               Image4;
  TImage               Image5;
  TImage               Image6;
  TImage               Image7;
  TImage               Image8;
  TImage                 * const code Screen2_Images[7]=
         {
         &Image2,
         &Image3,
         &Image4,
         &Image5,
         &Image6,
         &Image7,
         &Image8
         };
  TCImage                * const code Screen2_CImages[1]=
         {
         &Image1
         };



static void InitializeObjects() {
  Screen1.Color                     = 0x5AEB;
  Screen1.Width                     = 800;
  Screen1.Height                    = 480;
  Screen1.ImagesCount               = 0;
  Screen1.CImagesCount              = 0;
  Screen1.ObjectsCount              = 0;
  Screen1.OnSwipeUpPtr    = 0;
  Screen1.OnSwipeDownPtr  = 0;
  Screen1.OnSwipeLeftPtr  = 0;
  Screen1.OnSwipeRightPtr = 0;
  Screen1.OnZoomInPtr     = 0;
  Screen1.OnZoomOutPtr    = 0;

  Screen2.Color                     = 0x5AEB;
  Screen2.Width                     = 800;
  Screen2.Height                    = 480;
  Screen2.ImagesCount               = 7;
  Screen2.Images                    = Screen2_Images;
  Screen2.CImagesCount              = 1;
  Screen2.CImages                   = Screen2_CImages;
  Screen2.ObjectsCount              = 8;
  Screen2.OnSwipeUpPtr    = 0;
  Screen2.OnSwipeDownPtr  = 0;
  Screen2.OnSwipeLeftPtr  = 0;
  Screen2.OnSwipeRightPtr = 0;
  Screen2.OnZoomInPtr     = 0;
  Screen2.OnZoomOutPtr    = 0;


  Image2.OwnerScreen     = &Screen2;
  Image2.Order           = 1;
  Image2.Left            = 236;
  Image2.Top             = 72;
  Image2.Width           = 270;
  Image2.Height          = 80;
  Image2.Picture_Type    = 0;
  Image2.Picture_Ratio   = 1;
  Image2.Picture_Name    = time_update_bmp;
  Image2.Visible         = 1;
  Image2.Active          = 1;

  Image3.OwnerScreen     = &Screen2;
  Image3.Order           = 2;
  Image3.Left            = 273;
  Image3.Top             = 165;
  Image3.Width           = 252;
  Image3.Height          = 61;
  Image3.Picture_Type    = 0;
  Image3.Picture_Ratio   = 1;
  Image3.Picture_Name    = date_update_bmp;
  Image3.Visible         = 1;
  Image3.Active          = 1;

  Image4.OwnerScreen     = &Screen2;
  Image4.Order           = 3;
  Image4.Left            = 104;
  Image4.Top             = 337;
  Image4.Width           = 102;
  Image4.Height          = 64;
  Image4.Picture_Type    = 0;
  Image4.Picture_Ratio   = 1;
  Image4.Picture_Name    = hum_update_bmp;
  Image4.Visible         = 1;
  Image4.Active          = 1;

  Image5.OwnerScreen     = &Screen2;
  Image5.Order           = 4;
  Image5.Left            = 344;
  Image5.Top             = 338;
  Image5.Width           = 115;
  Image5.Height          = 60;
  Image5.Picture_Type    = 0;
  Image5.Picture_Ratio   = 1;
  Image5.Picture_Name    = temp_update_bmp;
  Image5.Visible         = 1;
  Image5.Active          = 1;

  Image6.OwnerScreen     = &Screen2;
  Image6.Order           = 5;
  Image6.Left            = 651;
  Image6.Top             = 339;
  Image6.Width           = 100;
  Image6.Height          = 64;
  Image6.Picture_Type    = 0;
  Image6.Picture_Ratio   = 1;
  Image6.Picture_Name    = ir_update_bmp;
  Image6.Visible         = 1;
  Image6.Active          = 1;

  Image7.OwnerScreen     = &Screen2;
  Image7.Order           = 6;
  Image7.Left            = 655;
  Image7.Top             = 195;
  Image7.Width           = 95;
  Image7.Height          = 78;
  Image7.Picture_Type    = 0;
  Image7.Picture_Ratio   = 1;
  Image7.Picture_Name    = uv_update_bmp;
  Image7.Visible         = 1;
  Image7.Active          = 1;

  Image8.OwnerScreen     = &Screen2;
  Image8.Order           = 7;
  Image8.Left            = 646;
  Image8.Top             = 72;
  Image8.Width           = 94;
  Image8.Height          = 69;
  Image8.Picture_Type    = 0;
  Image8.Picture_Ratio   = 1;
  Image8.Picture_Name    = lx_update_bmp;
  Image8.Visible         = 1;
  Image8.Active          = 1;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetImage(index)               CurrentScreen->Images[index]
#define GetCImage(index)              CurrentScreen->CImages[index]


void DrawImage(TImage *AImage) {
  if (AImage->Visible != 0) {
    TFT_Image(AImage->Left, AImage->Top, AImage->Picture_Name, AImage->Picture_Ratio);
  }
}

void DrawCImage(TCImage *AImage) {
  if (AImage->Visible != 0) {
    TFT_Image(AImage->Left, AImage->Top, AImage->Picture_Name, AImage->Picture_Ratio);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short cimage_idx;
  TCImage *local_cimage;
  char save_bled;

  object_pressed = 0;
  order = 0;
  image_idx = 0;
  cimage_idx = 0;
  CurrentScreen = aScreen;
  
  TFT_Set_DBC_SSD1963(0);

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init_SSD1963_Board_50(CurrentScreen->Width, CurrentScreen->Height);
    FT5XX6_SetSize(CurrentScreen->Width, CurrentScreen->Height);
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);

    while (order < CurrentScreen->ObjectsCount) {
    if (image_idx < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }
    
    if (cimage_idx < CurrentScreen->CImagesCount) {
      local_cimage = GetCImage(cimage_idx);
      if (order == local_cimage->Order) {
        cimage_idx++;
        order++;
        DrawCImage(local_cimage);
      }
    }
  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  image_order         = -1;
  cimage_order        = -1;
  //  Images
  for ( _object_count = 0 ; _object_count < CurrentScreen->ImagesCount ; _object_count++ ) {
    local_image = GetImage(_object_count);
    if (local_image->Active != 0) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }
  
  //  CImages
  for ( _object_count = 0 ; _object_count < CurrentScreen->CImagesCount ; _object_count++ ) {
    local_cimage = GetCImage(_object_count);
    if (local_cimage->Active != 0) {
      if (IsInsideObject(X, Y, local_cimage->Left, local_cimage->Top,
                         local_cimage->Width, local_cimage->Height) == 1) {
        cimage_order = local_cimage->Order;
        exec_cimage = local_cimage;
      }
    }
  }

  _object_count = -1;
  if (image_order >  _object_count )
    _object_count = image_order;
  if (cimage_order >  _object_count )
    _object_count = cimage_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_image          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == image_order) {
      if (exec_image->Active != 0) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  exec_image          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Images
    if (_object_count == image_order) {
      if (exec_image->Active != 0) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (void *)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_image          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == image_order) {
      if (exec_image->Active != 0) {
        PressedObject = (void *)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (FT5XX6_PressDetect()) {
    if (FT5XX6_GetCoordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU()
{  // If bus is busy wait SDA high before initializing I2C module
  GPIO_Digital_Output(&GPIOF_BASE, _GPIO_PINMASK_1);
  GPIO_Digital_Input(&GPIOF_BASE, _GPIO_PINMASK_0);
  GPIOF_ODR.B1 = 1;
  while (GPIOF_IDR.B0 == 0) {
    GPIOF_ODR.B1 = 0;
    Delay_us(10);
    GPIOF_ODR.B1 = 1;
    Delay_us(10);
  }
  I2C2_Init_Advanced(400000, &_GPIO_MODULE_I2C2_PF01);

  TFT_Set_Default_Mode();
}

char FT5XX6_Config()  {
  FT5XX6_SetI2CAddress(0x38);
  if (FT5XX6_IsOperational() != FT5XX6_OK)
    return  FT5XX6_FAILURE;
  FT5XX6_SetDefaultMode();
  FT5XX6_SetController(_TC_FT5X06);
  FT5XX6_SetSizeAndRotation(800,480,0);
return FT5XX6_OK;
}

void Start_TP() {
  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}